@@ -46,6 +46,7 @@ class DeepsetCloudDocumentStore(KeywordDocumentStore):
         return_embedding: bool = False,
         label_index: str = "default",
         embedding_dim: int = 768,
+        use_prefiltering: bool = False,
     ):
         """
         A DocumentStore facade enabling you to interact with the documents stored in deepset Cloud.
@@ -86,6 +87,9 @@ class DeepsetCloudDocumentStore(KeywordDocumentStore):
         :param label_index: index for the evaluation set interface
         :param return_embedding: To return document embedding.
         :param embedding_dim: Specifies the dimensionality of the embedding vector (only needed when using a dense retriever, for example, DensePassageRetriever pr EmbeddingRetriever, on top).
+        :param use_prefiltering: By default, DeepsetCloudDocumentStore uses post-filtering when querying with filters.
+                                 To use pre-filtering instead, set this parameter to `True`. Note that pre-filtering
+                                 comes at the cost of higher latency.
         """
         self.index = index
         self.label_index = label_index
@@ -93,6 +97,7 @@ class DeepsetCloudDocumentStore(KeywordDocumentStore):
         self.similarity = similarity
         self.return_embedding = return_embedding
         self.embedding_dim = embedding_dim
+        self.use_prefiltering = use_prefiltering
         self.client = DeepsetCloud.get_index_client(
             api_key=api_key, api_endpoint=api_endpoint, workspace=workspace, index=index
         )
@@ -118,23 +123,25 @@ class DeepsetCloudDocumentStore(KeywordDocumentStore):
             indexing_info = index_info["indexing"]
             if indexing_info["pending_file_count"] > 0:
                 logger.warning(
-                    f"{indexing_info['pending_file_count']} files are pending to be indexed. "
-                    f"Indexing status: {indexing_info['status']}"
+                    "%s files are pending to be indexed. Indexing status: %s",
+                    indexing_info["pending_file_count"],
+                    indexing_info["status"],
                 )
             if index in deployed_unhealthy_pipelines:
                 logger.warning(
-                    f"The index '{index}' is unhealthy and should be redeployed using "
-                    f"`Pipeline.undeploy_on_deepset_cloud()` and `Pipeline.deploy_on_deepset_cloud()`."
+                    "The index '%s' is unhealthy and should be redeployed using "
+                    "`Pipeline.undeploy_on_deepset_cloud()` and `Pipeline.deploy_on_deepset_cloud()`.",
+                    index,
                 )
         else:
             logger.info(
-                f"You are using a DeepsetCloudDocumentStore with an index that does not exist on deepset Cloud. "
-                f"This document store always returns empty responses. This can be useful if you want to "
-                f"create a new pipeline within deepset Cloud.\n"
-                f"In order to create a new pipeline on deepset Cloud, take the following steps: \n"
-                f"  - create query and indexing pipelines using this DocumentStore\n"
-                f"  - call `Pipeline.save_to_deepset_cloud()` passing the pipelines and a `pipeline_config_name`\n"
-                f"  - call `Pipeline.deploy_on_deepset_cloud()` passing the `pipeline_config_name`"
+                "You are using a DeepsetCloudDocumentStore with an index that does not exist on deepset Cloud. "
+                "This document store always returns empty responses. This can be useful if you want to "
+                "create a new pipeline within deepset Cloud.\n"
+                "In order to create a new pipeline on deepset Cloud, take the following steps: \n"
+                "  - create query and indexing pipelines using this DocumentStore\n"
+                "  - call `Pipeline.save_to_deepset_cloud()` passing the pipelines and a `pipeline_config_name`\n"
+                "  - call `Pipeline.deploy_on_deepset_cloud()` passing the `pipeline_config_name`"
             )
 
         self.evaluation_set_client = DeepsetCloud.get_evaluation_set_client(
@@ -417,6 +424,7 @@ class DeepsetCloudDocumentStore(KeywordDocumentStore):
             index=index,
             scale_score=scale_score,
             headers=headers,
+            use_prefiltering=self.use_prefiltering,
         )
         docs = [Document.from_dict(doc) for doc in doc_dicts]
         return docs
@@ -553,18 +561,19 @@ class DeepsetCloudDocumentStore(KeywordDocumentStore):
                 )
         else:
             filters = [filters] * len(queries) if filters is not None else [{}] * len(queries)
-            for query, cur_filters in zip(queries, filters):
-                cur_docs = self.query(
-                    query=query,
-                    filters=cur_filters,
-                    top_k=top_k,
-                    custom_query=custom_query,
-                    index=index,
-                    headers=headers,
-                    all_terms_must_match=all_terms_must_match,
-                    scale_score=scale_score,
-                )
-                documents.append(cur_docs)
+
+        for query, cur_filters in zip(queries, filters):
+            cur_docs = self.query(
+                query=query,
+                filters=cur_filters,
+                top_k=top_k,
+                custom_query=custom_query,
+                index=index,
+                headers=headers,
+                all_terms_must_match=all_terms_must_match,
+                scale_score=scale_score,
+            )
+            documents.append(cur_docs)
 
         return documents
 
