@@ -150,7 +150,7 @@ class DataSilo:
         :return: None
         """
 
-        logger.info("\nLoading data into the data silo ..." "{}".format(TRACTOR_SMALL))
+        logger.info("\nLoading data into the data silo ... %s", TRACTOR_SMALL)
         # train data
         logger.info("LOADING TRAIN DATA")
         logger.info("==================")
@@ -161,7 +161,7 @@ class DataSilo:
         elif self.processor.train_filename:
             # or from a file (default)
             train_file = self.processor.data_dir / self.processor.train_filename
-            logger.info("Loading train set from: {} ".format(train_file))
+            logger.info("Loading train set from: %s ", train_file)
             self.data["train"], self.tensor_names = self._get_dataset(train_file)
         else:
             logger.info("No train set is being loaded")
@@ -178,7 +178,7 @@ class DataSilo:
         elif self.processor.dev_filename:
             # or from file (default)
             dev_file = self.processor.data_dir / self.processor.dev_filename
-            logger.info("Loading dev set from: {}".format(dev_file))
+            logger.info("Loading dev set from: %s", dev_file)
             self.data["dev"], _ = self._get_dataset(dev_file)
         elif self.processor.dev_split > 0.0:
             # or split it apart from train set
@@ -199,7 +199,7 @@ class DataSilo:
         elif self.processor.test_filename:
             # or from file (default)
             test_file = self.processor.data_dir / self.processor.test_filename
-            logger.info("Loading test set from: {}".format(test_file))
+            logger.info("Loading test set from: %s", test_file)
             if self.tensor_names:
                 self.data["test"], _ = self._get_dataset(test_file)
             else:
@@ -336,7 +336,9 @@ class DataSilo:
             logger.warning("No dev set created. Please adjust the dev_split parameter.")
 
         logger.info(
-            f"Took {len(dev_dataset)} samples out of train set to create dev set (dev split is roughly {self.processor.dev_split})"
+            "Took %s samples out of train set to create dev set (dev split is roughly %s)",
+            len(dev_dataset),
+            self.processor.dev_split,
         )
 
     def random_split_ConcatDataset(self, ds: ConcatDataset, lengths: List[int]):
@@ -387,7 +389,7 @@ class DataSilo:
                 clipped, ave_len, seq_lens, max_seq_len = self._calc_length_stats_biencoder()
             else:
                 logger.warning(
-                    f"Could not compute length statistics because 'input_ids' or 'query_input_ids' and 'passage_input_ids' are missing."
+                    "Could not compute length statistics because 'input_ids' or 'query_input_ids' and 'passage_input_ids' are missing."
                 )
                 clipped = -1
                 ave_len = -1
@@ -404,40 +406,43 @@ class DataSilo:
         else:
             self.counts["test"] = 0
 
-        logger.info("Examples in train: {}".format(self.counts["train"]))
-        logger.info("Examples in dev  : {}".format(self.counts["dev"]))
-        logger.info("Examples in test : {}".format(self.counts["test"]))
-        logger.info("Total examples   : {}".format(self.counts["train"] + self.counts["dev"] + self.counts["test"]))
+        logger.info("Examples in train: %s", self.counts["train"])
+        logger.info("Examples in dev  : %s", self.counts["dev"])
+        logger.info("Examples in test : %s", self.counts["test"])
+        logger.info("Total examples   : %s", self.counts["train"] + self.counts["dev"] + self.counts["test"])
         logger.info("")
         if self.data["train"]:
             if "input_ids" in self.tensor_names:
-                logger.info("Longest sequence length observed after clipping:     {}".format(max(seq_lens)))
-                logger.info("Average sequence length after clipping: {}".format(ave_len))
-                logger.info("Proportion clipped:      {}".format(clipped))
+                logger.info("Longest sequence length observed after clipping:     %s", max(seq_lens))
+                logger.info("Average sequence length after clipping: %s", ave_len)
+                logger.info("Proportion clipped:      %s", clipped)
                 if clipped > 0.5:
                     logger.info(
-                        f"[Haystack Tip] {round(clipped * 100, 1)}% of your samples got cut down to {max_seq_len} tokens. "
+                        "[Haystack Tip] %s%% of your samples got cut down to %s tokens. "
                         "Consider increasing max_seq_len "
-                        f"(the maximum value allowed with the current model is max_seq_len={self.processor.tokenizer.model_max_length}, "
+                        "(the maximum value allowed with the current model is max_seq_len=%s, "
                         "if this is not enough consider splitting the document in smaller units or changing the model). "
-                        "This will lead to higher memory consumption but is likely to improve your model performance"
+                        "This will lead to higher memory consumption but is likely to improve your model performance",
+                        round(clipped * 100, 1),
+                        max_seq_len,
+                        self.processor.tokenizer.model_max_length,
                     )
             elif "query_input_ids" in self.tensor_names and "passage_input_ids" in self.tensor_names:
                 logger.info(
-                    "Longest query length observed after clipping: {}   - for max_query_len: {}".format(
-                        max(seq_lens[0]), max_seq_len[0]
-                    )
+                    "Longest query length observed after clipping: %s   - for max_query_len: %s",
+                    max(seq_lens[0]),
+                    max_seq_len[0],
                 )
-                logger.info("Average query length after clipping:          {}".format(ave_len[0]))
-                logger.info("Proportion queries clipped:                   {}".format(clipped[0]))
+                logger.info("Average query length after clipping:          %s", ave_len[0])
+                logger.info("Proportion queries clipped:                   %s", clipped[0])
                 logger.info("")
                 logger.info(
-                    "Longest passage length observed after clipping: {}   - for max_passage_len: {}".format(
-                        max(seq_lens[1]), max_seq_len[1]
-                    )
+                    "Longest passage length observed after clipping: %s   - for max_passage_len: %s",
+                    max(seq_lens[1]),
+                    max_seq_len[1],
                 )
-                logger.info("Average passage length after clipping:          {}".format(ave_len[1]))
-                logger.info("Proportion passages clipped:                    {}".format(clipped[1]))
+                logger.info("Average passage length after clipping:          %s", ave_len[1])
+                logger.info("Proportion passages clipped:                    %s", clipped[1])
 
         tracker.track_params(
             {
@@ -524,20 +529,20 @@ class DataSiloForCrossVal:
         self.batch_size = origsilo.batch_size
         # should not be necessary, xval makes no sense with huge data
         # sampler_train = DistributedSampler(self.data["train"])
-        sampler_train = RandomSampler(trainset)
+        sampler_train = RandomSampler(trainset)  # type: ignore [arg-type]
 
         self.data_loader_train = NamedDataLoader(
-            dataset=trainset, sampler=sampler_train, batch_size=self.batch_size, tensor_names=self.tensor_names
+            dataset=trainset, sampler=sampler_train, batch_size=self.batch_size, tensor_names=self.tensor_names  # type: ignore [arg-type]
         )
         self.data_loader_dev = NamedDataLoader(
-            dataset=devset,
-            sampler=SequentialSampler(devset),
+            dataset=devset,  # type: ignore [arg-type]
+            sampler=SequentialSampler(devset),  # type: ignore [arg-type]
             batch_size=self.batch_size,
             tensor_names=self.tensor_names,
         )
         self.data_loader_test = NamedDataLoader(
-            dataset=testset,
-            sampler=SequentialSampler(testset),
+            dataset=testset,  # type: ignore [arg-type]
+            sampler=SequentialSampler(testset),  # type: ignore [arg-type]
             batch_size=self.batch_size,
             tensor_names=self.tensor_names,
         )
@@ -550,7 +555,7 @@ class DataSiloForCrossVal:
     def make(
         cls,
         datasilo: DataSilo,
-        sets: List[str] = ["train", "dev", "test"],
+        sets: Optional[List[str]] = None,
         n_splits: int = 5,
         shuffle: bool = True,
         random_state: Optional[int] = None,
@@ -563,7 +568,7 @@ class DataSiloForCrossVal:
         original data silo passed on.
 
         :param datasilo: The data silo that contains the original data.
-        :param sets: Which sets to use to create the xval folds (strings)
+        :param sets: Which sets to use to create the xval folds (strings). By default, "train", "dev", and "test" are used.
         :param n_splits: number of folds to create
         :param shuffle: shuffle each class' samples before splitting
         :param random_state: random state for shuffling
@@ -571,6 +576,8 @@ class DataSiloForCrossVal:
             It is never done with question answering.
         :param n_neg_answers_per_question: number of negative answers per question to include for training
         """
+        if sets is None:
+            sets = ["train", "dev", "test"]
         if "question_answering" in datasilo.processor.tasks and n_inner_splits is None:  # type: ignore
             return cls._make_question_answering(
                 datasilo, sets, n_splits, shuffle, random_state, n_neg_answers_per_question
@@ -583,7 +590,7 @@ class DataSiloForCrossVal:
     def _make_question_answering(
         cls,
         datasilo: DataSilo,
-        sets: List[str] = ["train", "dev", "test"],
+        sets: Optional[List[str]] = None,
         n_splits: int = 5,
         shuffle: bool = True,
         random_state: Optional[int] = None,
@@ -595,12 +602,14 @@ class DataSiloForCrossVal:
         data for question-answering-
 
         :param datasilo: The data silo that contains the original data.
-        :param sets: Which sets to use to create the xval folds (strings).
+        :param sets: Which sets to use to create the xval folds (strings). By default, "train", "dev", and "test" are used.
         :param n_splits: Number of folds to create.
         :param shuffle: Shuffle each class' samples before splitting.
         :param random_state: Random state for shuffling.
         :param n_neg_answers_per_question: Number of negative answers per question to include for training.
         """
+        if sets is None:
+            sets = ["train", "dev", "test"]
         assert "id" in datasilo.tensor_names, f"Expected tensor 'id' in tensor names, found {datasilo.tensor_names}"  # type: ignore
         assert "labels" in datasilo.tensor_names, f"Expected tensor 'labels' in tensor names, found {datasilo.tensor_names}"  # type: ignore
 
@@ -616,7 +625,7 @@ class DataSiloForCrossVal:
         documents = []
         keyfunc = lambda x: x[id_index][0]  # pylint: disable=unnecessary-lambda-assignment
         all_data = sorted(all_data.datasets, key=keyfunc)  # type: ignore
-        for key, document in groupby(all_data, key=keyfunc):  # type: ignore
+        for _, document in groupby(all_data, key=keyfunc):  # type: ignore
             documents.append(list(document))
 
         xval_split = cls._split_for_qa(
@@ -642,7 +651,7 @@ class DataSiloForCrossVal:
             for doc in actual_train_set:
                 keyfunc = lambda x: x[id_index][1]  # pylint: disable=unnecessary-lambda-assignment
                 doc = sorted(doc, key=keyfunc)
-                for key, question in groupby(doc, key=keyfunc):
+                for _, question in groupby(doc, key=keyfunc):
                     # add all available answrs to train set
                     sample_list = list(question)
                     neg_answer_idx: List[int] = []
@@ -664,7 +673,7 @@ class DataSiloForCrossVal:
 
             ds_train = train_samples
             ds_test = [sample for document in test_set for sample in document]
-            silos.append(DataSiloForCrossVal(datasilo, ds_train, ds_dev, ds_test))
+            silos.append(DataSiloForCrossVal(datasilo, ds_train, ds_dev, ds_test))  # type: ignore [arg-type]
         return silos
 
     @staticmethod
@@ -685,7 +694,7 @@ class DataSiloForCrossVal:
             questions_per_doc.append(len(questions))
 
         # split documents into n_splits splits with approximately same number of questions per split
-        questions_per_doc = np.array(questions_per_doc)
+        questions_per_doc = np.array(questions_per_doc)  # type: ignore [assignment]
         accumulated_questions_per_doc = questions_per_doc.cumsum()  # type: ignore
         questions_per_fold = accumulated_questions_per_doc[-1] // n_splits
         accumulated_questions_per_fold = np.array(range(1, n_splits)) * questions_per_fold
@@ -701,7 +710,7 @@ class DataSiloForCrossVal:
 
         for idx, split in enumerate(splits):
             current_test_set = split
-            current_train_set = np.hstack(np.delete(splits, idx, axis=0))
+            current_train_set = np.hstack(np.delete(splits, idx, axis=0))  # type: ignore [call-overload]
 
             yield current_train_set, current_test_set
 
