@@ -38,17 +38,18 @@ def sample_to_features_text(sample, tasks, max_seq_len, tokenizer):
 
         if (len(inputs["input_ids"]) - inputs["special_tokens_mask"].count(1)) != len(sample.tokenized["tokens"]):
             logger.error(
-                f"FastTokenizer encoded sample {sample.clear_text['text']} to "
-                f"{len(inputs['input_ids']) - inputs['special_tokens_mask'].count(1)} tokens, which differs "
-                f"from number of tokens produced in tokenize_with_metadata(). \n"
-                f"Further processing is likely to be wrong."
+                "FastTokenizer encoded sample %s to %s tokens, which differs "
+                "from number of tokens produced in tokenize_with_metadata(). \n"
+                "Further processing is likely to be wrong.",
+                sample.clear_text["text"],
+                len(inputs["input_ids"]) - inputs["special_tokens_mask"].count(1),
             )
     else:
         # TODO It might be cleaner to adjust the data structure in sample.tokenized
         tokens_a = sample.tokenized["tokens"]
         tokens_b = sample.tokenized.get("tokens_b", None)
 
-        inputs = tokenizer.encode_plus(
+        inputs = tokenizer(
             tokens_a,
             tokens_b,
             add_special_tokens=True,
@@ -93,7 +94,7 @@ def sample_to_features_text(sample, tasks, max_seq_len, tokenizer):
                 # id of label
                 try:
                     label_ids = [label_list.index(label_raw)]
-                except ValueError as e:
+                except ValueError:
                     raise ValueError(f"[Task: {task_name}] Observed label {label_raw} not in defined label_list")
             elif task["task_type"] == "multilabel_classification":
                 # multi-hot-format
