@@ -114,7 +114,7 @@ class LanguageModel(nn.Module, ABC):
                 if value:
                     self._output_dims = value
                     return value
-            except AttributeError as e:
+            except AttributeError:
                 raise ModelingError("Can't get the output dimension before loading the model.")
 
         raise ModelingError("Could not infer the output dimensions of the language model.")
@@ -124,10 +124,10 @@ class LanguageModel(nn.Module, ABC):
         Save the configuration of the language model in Haystack format.
         """
         save_filename = Path(save_dir) / "language_model_config.json"
-        setattr(self.model.config, "name", self.name)
-        setattr(self.model.config, "language", self.language)
+        setattr(self.model.config, "name", self.name)  # type: ignore [union-attr]
+        setattr(self.model.config, "language", self.language)  # type: ignore [union-attr]
 
-        string = self.model.config.to_json_string()
+        string = self.model.config.to_json_string()  # type: ignore [union-attr,operator]
         with open(save_filename, "w") as file:
             file.write(string)
 
@@ -143,7 +143,7 @@ class LanguageModel(nn.Module, ABC):
         model_to_save = self.model.module if hasattr(self.model, "module") else self.model  # Only save the model itself
 
         if not state_dict:
-            state_dict = model_to_save.state_dict()
+            state_dict = model_to_save.state_dict()  # type: ignore [union-attr]
         torch.save(state_dict, save_name)
         self.save_config(save_dir)
 
@@ -191,11 +191,11 @@ class LanguageModel(nn.Module, ABC):
 
         elif self.extraction_strategy == "reduce_mean":
             vecs = self._pool_tokens(
-                sequence_output, padding_mask, self.extraction_strategy, ignore_first_token=ignore_first_token
+                sequence_output, padding_mask, self.extraction_strategy, ignore_first_token=ignore_first_token  # type: ignore [arg-type]   # type: ignore [arg-type]
             )
         elif self.extraction_strategy == "reduce_max":
             vecs = self._pool_tokens(
-                sequence_output, padding_mask, self.extraction_strategy, ignore_first_token=ignore_first_token
+                sequence_output, padding_mask, self.extraction_strategy, ignore_first_token=ignore_first_token  # type: ignore [arg-type]   # type: ignore [arg-type]
             )
         elif self.extraction_strategy == "cls_token":
             vecs = sequence_output[:, 0, :].cpu().numpy()
@@ -293,14 +293,14 @@ class HFLanguageModel(LanguageModel):
             model_emb_size = self.model.resize_token_embeddings(new_num_tokens=None).num_embeddings
             vocab_size = model_emb_size + n_added_tokens
             logger.info(
-                f"Resizing embedding layer of LM from {model_emb_size} to {vocab_size} to cope with custom vocab."
+                "Resizing embedding layer of LM from %s to %s to cope with custom vocab.", model_emb_size, vocab_size
             )
             self.model.resize_token_embeddings(vocab_size)
             # verify
             model_emb_size = self.model.resize_token_embeddings(new_num_tokens=None).num_embeddings
             assert vocab_size == model_emb_size
 
-    def forward(
+    def forward(  # type: ignore [override]
         self,
         input_ids: torch.Tensor,
         attention_mask: torch.Tensor,
@@ -337,9 +337,9 @@ class HFLanguageModel(LanguageModel):
         if attention_mask is not None:
             params["attention_mask"] = attention_mask
         if output_hidden_states:
-            params["output_hidden_states"] = output_hidden_states
+            params["output_hidden_states"] = output_hidden_states  # type: ignore [assignment]
         if output_attentions:
-            params["output_attentions"] = output_attentions
+            params["output_attentions"] = output_attentions  # type: ignore [assignment]
 
         return self.model(**params, return_dict=return_dict)
 
@@ -424,7 +424,7 @@ class HFLanguageModelWithPooler(HFLanguageModel):
         """
         output_tuple = super().forward(
             input_ids=input_ids,
-            segment_ids=segment_ids,
+            segment_ids=segment_ids,  # type: ignore [arg-type]
             attention_mask=attention_mask,
             output_hidden_states=output_hidden_states,
             output_attentions=output_attentions,
@@ -464,7 +464,7 @@ class HFLanguageModelNoSegmentIds(HFLanguageModelWithPooler):
             specified using the arguments `output_hidden_states` and `output_attentions`.
         """
         if segment_ids is not None:
-            logger.warning(f"'segment_ids' is not None, but %s does not use them. They will be ignored.", self.name)
+            logger.warning("'segment_ids' is not None, but %s does not use them. They will be ignored.", self.name)
 
         return super().forward(
             input_ids=input_ids,
@@ -515,7 +515,7 @@ class DPREncoder(LanguageModel):
         model_classname = f"DPR{self.role.capitalize()}Encoder"
         try:
             model_class: Type[PreTrainedModel] = getattr(transformers, model_classname)
-        except AttributeError as e:
+        except AttributeError:
             raise ModelingError(f"Model class of type '{model_classname}' not found.")
 
         haystack_lm_config = Path(pretrained_model_name_or_path) / "language_model_config.json"
@@ -636,8 +636,9 @@ class DPREncoder(LanguageModel):
         """
         if model_config.model_type.lower() != "bert":
             logger.warning(
-                f"Using a model of type '{model_config.model_type}' which might be incompatible with DPR encoders. "
-                f"Only Bert-based encoders are supported. They need input_ids, token_type_ids, attention_mask as input tensors."
+                "Using a model of type '%s' which might be incompatible with DPR encoders. "
+                "Only Bert-based encoders are supported. They need input_ids, token_type_ids, attention_mask as input tensors.",
+                model_config.model_type,
             )
         config_dict = vars(model_config)
         if model_kwargs:
@@ -704,9 +705,9 @@ class DPREncoder(LanguageModel):
         :param input_ids: The IDs of each token in the input sequence. It's a tensor of shape [batch_size, number_of_hard_negative, max_seq_len].
         :param segment_ids: The ID of the segment. For example, in next sentence prediction, the tokens in the
            first sentence are marked with 0 and the tokens in the second sentence are marked with 1.
-           It is a tensor of shape [batch_size, number_of_hard_negative_passages, max_seq_len].
+           It is a tensor of shape [batch_size, max_seq_len].
         :param attention_mask: A mask that assigns 1 to valid input tokens and 0 to padding tokens
-           of shape [batch_size,  number_of_hard_negative_passages, max_seq_len].
+           of shape [batch_size, max_seq_len].
         :param output_hidden_states: whether to add the hidden states along with the pooled output
         :param output_attentions: unused
         :return: Embeddings for each token in the input sequence.
@@ -862,7 +863,8 @@ def get_language_model(
     config_file_exists = os.path.exists(config_file)
     if config_file_exists:
         # it's a local directory in Haystack format
-        config = json.load(open(config_file))
+        with open(config_file) as f:
+            config = json.load(f)
         model_type = config["name"]
 
     if not model_type:
@@ -875,12 +877,13 @@ def get_language_model(
 
     if not model_type:
         logger.error(
-            f"Model type not understood for '{pretrained_model_name_or_path}' "
-            f"({model_type if model_type else 'model_type not set'}). "
+            "Model type not understood for '%s' (%s). "
             "Either supply the local path for a saved model, "
             "or the name of a model that can be downloaded from the Model Hub. "
             "Ensure that the model class name can be inferred from the directory name "
-            "when loading a Transformers model."
+            "when loading a Transformers model.",
+            pretrained_model_name_or_path,
+            model_type if model_type else "model_type not set",
         )
         logger.error("Using the AutoModel class for '%s'. This can cause crashes!", pretrained_model_name_or_path)
         model_type = "Auto"
@@ -956,7 +959,7 @@ def _get_model_type(
 
     if model_type and model_type.lower() == "roberta" and "mlm" in model_name_or_path.lower():
         logger.error(
-            f"MLM part of codebert is currently not supported in Haystack: '{model_name_or_path}' may crash later."
+            "MLM part of codebert is currently not supported in Haystack: '%s' may crash later.", model_name_or_path
         )
 
     return model_type
