@@ -5,8 +5,10 @@ import os
 import pickle
 import random
 import signal
+from functools import wraps
 from copy import deepcopy
 from itertools import islice
+
 import numpy as np
 import torch
 import torch.distributed as dist
@@ -28,6 +30,29 @@ class GracefulKiller:
         self.kill_now = True
 
 
+def silence_transformers_logs(from_pretrained_func):
+    """
+    A wrapper that raises the log level of Transformers to
+    ERROR to hide some unnecessary warnings.
+    """
+
+    @wraps(from_pretrained_func)
+    def quiet_from_pretrained_func(cls, *args, **kwargs):
+        # Raise the log level of Transformers
+        t_logger = logging.getLogger("transformers")
+        original_log_level = t_logger.level
+        t_logger.setLevel(logging.ERROR)
+
+        result = from_pretrained_func(cls, *args, **kwargs)
+
+        # Restore the log level
+        t_logger.setLevel(original_log_level)
+
+        return result
+
+    return quiet_from_pretrained_func
+
+
 def set_all_seeds(seed: int, deterministic_cudnn: bool = False) -> None:
     """
     Setting multiple seeds to make runs reproducible.
@@ -36,7 +61,7 @@ def set_all_seeds(seed: int, deterministic_cudnn: bool = False) -> None:
     but might slow down your training (see https://pytorch.org/docs/stable/notes/randomness.html#cudnn) !
 
     :param seed:number to use as seed
-    :param deterministic_torch: Enable for full reproducibility when using CUDA. Caution: might slow down training.
+    :param deterministic_cudnn: Enable for full reproducibility when using CUDA. Caution: might slow down training.
     """
     random.seed(seed)
     np.random.seed(seed)
@@ -52,7 +77,7 @@ def initialize_device_settings(
     use_cuda: Optional[bool] = None,
     local_rank: int = -1,
     multi_gpu: bool = True,
-    devices: List[Union[str, torch.device]] = None,
+    devices: Optional[List[Union[str, torch.device]]] = None,
 ) -> Tuple[List[torch.device], int]:
     """
     Returns a list of available devices.
@@ -77,7 +102,7 @@ def initialize_device_settings(
             torch_devices: List[torch.device] = [torch.device(device) for device in devices]
             devices_to_use = torch_devices
         else:
-            devices_to_use = devices
+            devices_to_use = devices  # type: ignore [assignment]
         n_gpu = sum(1 for device in devices_to_use if "cpu" not in device.type)
     elif local_rank == -1:
         if torch.cuda.is_available():
@@ -242,6 +267,7 @@ def grouper(iterable, n: int, worker_id: int = 0, total_workers: int = 1):
     :param worker_id: the worker_id for the PyTorch DataLoader
     :param total_workers: total number of workers for the PyTorch DataLoader
     """
+
     # TODO make me comprehensible :)
     def get_iter_start_pos(gen):
         start_pos = worker_id * n
@@ -302,7 +328,7 @@ def log_ascii_workers(n, logger):
     f_worker_lines = WORKER_F.split("\n")
     x_worker_lines = WORKER_X.split("\n")
     all_worker_lines = []
-    for i in range(n):
+    for _ in range(n):
         rand = np.random.randint(low=0, high=3)
         if rand % 3 == 0:
             all_worker_lines.append(f_worker_lines)
