@@ -1,4 +1,3 @@
-from abc import abstractmethod
 from typing import List, Optional, Tuple, Dict, Union
 
 try:
@@ -117,8 +116,9 @@ class TableReader(BaseReader):
         self.devices, _ = initialize_device_settings(devices=devices, use_cuda=use_gpu, multi_gpu=False)
         if len(self.devices) > 1:
             logger.warning(
-                f"Multiple devices are not supported in {self.__class__.__name__} inference, "
-                f"using the first device {self.devices[0]}."
+                "Multiple devices are not supported in %s inference, using the first device %s.",
+                self.__class__.__name__,
+                self.devices[0],
             )
 
         config = TapasConfig.from_pretrained(model_name_or_path, use_auth_token=use_auth_token)
@@ -185,7 +185,7 @@ class TableReader(BaseReader):
         Use loaded TableQA model to find answers for the supplied queries in the supplied Documents
         of content_type ``'table'``.
 
-        Returns dictionary containing query and list of Answer objects sorted by (desc.) score.
+        Returns dictionary containing query and list of Answer objects sorted by (descending) score.
         WARNING: The answer scores are not reliable, as they are always extremely high, even if
         a question cannot be answered by a given table.
 
@@ -205,89 +205,29 @@ class TableReader(BaseReader):
         :param top_k: The maximum number of answers to return per query.
         :param batch_size: Not applicable.
         """
-        results: Dict = {"queries": queries, "answers": []}
+        if top_k is None:
+            top_k = self.top_k
 
-        single_doc_list = False
-        # Docs case 1: single list of Documents -> apply each query to all Documents
-        if len(documents) > 0 and isinstance(documents[0], Document):
-            single_doc_list = True
-            for query in queries:
-                for doc in documents:
-                    if not isinstance(doc, Document):
-                        raise HaystackError(f"doc was of type {type(doc)}, but expected a Document.")
-                    preds = self.predict(query=query, documents=[doc], top_k=top_k)
-                    results["answers"].append(preds["answers"])
+        single_doc_list = bool(len(documents) > 0 and isinstance(documents[0], Document))
 
-        # Docs case 2: list of lists of Documents -> apply each query to corresponding list of Documents, if queries
-        # contains only one query, apply it to each list of Documents
-        elif len(documents) > 0 and isinstance(documents[0], list):
-            if len(queries) == 1:
-                queries = queries * len(documents)
-            if len(queries) != len(documents):
-                raise HaystackError("Number of queries must be equal to number of provided Document lists.")
-            for query, cur_docs in zip(queries, documents):
-                if not isinstance(cur_docs, list):
-                    raise HaystackError(f"cur_docs was of type {type(cur_docs)}, but expected a list of Documents.")
-                preds = self.predict(query=query, documents=cur_docs, top_k=top_k)
-                results["answers"].append(preds["answers"])
+        inputs = _flatten_inputs(queries, documents)
+        results: Dict = self.table_encoder.predict_batch(
+            queries=inputs["queries"], documents=inputs["docs"], top_k=top_k
+        )
 
         # Group answers by question in case of multiple queries and single doc list
         if single_doc_list and len(queries) > 1:
-            answers_per_query = int(len(results["answers"]) / len(queries))
+            num_docs_per_query = int(len(results["answers"]) / len(queries))
             answers = []
-            for i in range(0, len(results["answers"]), answers_per_query):
-                answer_group = results["answers"][i : i + answers_per_query]
+            for i in range(0, len(results["answers"]), num_docs_per_query):
+                answer_group = results["answers"][i : i + num_docs_per_query]
                 answers.append(answer_group)
             results["answers"] = answers
 
         return results
 
 
-class _BaseTapasEncoder:
-    @staticmethod
-    def _calculate_answer_offsets(answer_coordinates: List[Tuple[int, int]], table: pd.DataFrame) -> List[Span]:
-        """
-        Calculates the answer cell offsets of the linearized table based on the answer cell coordinates.
-        """
-        answer_offsets = []
-        n_rows, n_columns = table.shape
-        for coord in answer_coordinates:
-            answer_cell_offset = (coord[0] * n_columns) + coord[1]
-            answer_offsets.append(Span(start=answer_cell_offset, end=answer_cell_offset + 1))
-        return answer_offsets
-
-    @staticmethod
-    def _check_documents(documents: List[Document]) -> List[Document]:
-        table_documents = []
-        for document in documents:
-            if document.content_type != "table":
-                logger.warning("Skipping document with id '%s' in TableReader as it is not of type table.", document.id)
-                continue
-
-            table: pd.DataFrame = document.content
-            if table.shape[0] == 0:
-                logger.warning(
-                    "Skipping document with id '%s' in TableReader as it does not contain any rows.", document.id
-                )
-                continue
-
-            table_documents.append(document)
-        return table_documents
-
-    @staticmethod
-    def _preprocess(query: str, table: pd.DataFrame, tokenizer, max_seq_len) -> BatchEncoding:
-        """Tokenize the query and table."""
-        model_inputs = tokenizer(
-            table=table, queries=query, max_length=max_seq_len, return_tensors="pt", truncation=True
-        )
-        return model_inputs
-
-    @abstractmethod
-    def predict(self, query: str, documents: List[Document], top_k: int) -> Dict:
-        pass
-
-
-class _TapasEncoder(_BaseTapasEncoder):
+class _TapasEncoder:
     def __init__(
         self,
         device: torch.device,
@@ -308,10 +248,12 @@ class _TapasEncoder(_BaseTapasEncoder):
         self.device = device
 
     def _predict_tapas(self, inputs: BatchEncoding, document: Document) -> Answer:
-        table: pd.DataFrame = document.content
+        orig_table: pd.DataFrame = document.content
+        string_table = orig_table.astype(str)
 
         # Forward query and table through model and convert logits to predictions
-        with torch.no_grad():
+        self.model.eval()
+        with torch.inference_mode():
             outputs = self.model(**inputs)
 
         inputs.to("cpu")
@@ -331,7 +273,7 @@ class _TapasEncoder(_BaseTapasEncoder):
         current_answer_coordinates = predicted_answer_coordinates[0]
         current_answer_cells = []
         for coordinate in current_answer_coordinates:
-            current_answer_cells.append(table.iat[coordinate])
+            current_answer_cells.append(string_table.iat[coordinate])
 
         # Get aggregation operator
         if self.model.config.aggregation_labels is not None:
@@ -347,16 +289,16 @@ class _TapasEncoder(_BaseTapasEncoder):
         else:
             answer_str = self._aggregate_answers(current_aggregation_operator, current_answer_cells)
 
-        answer_offsets = self._calculate_answer_offsets(current_answer_coordinates, document.content)
+        answer_offsets = _calculate_answer_offsets(current_answer_coordinates, string_table)
 
         answer = Answer(
             answer=answer_str,
             type="extractive",
             score=current_score,
-            context=document.content,
+            context=string_table,
             offsets_in_document=answer_offsets,
             offsets_in_context=answer_offsets,
-            document_id=document.id,
+            document_ids=[document.id],
             meta={"aggregation_operator": current_aggregation_operator, "answer_cells": current_answer_cells},
         )
         return answer
@@ -366,8 +308,9 @@ class _TapasEncoder(_BaseTapasEncoder):
     ) -> float:
         # Calculate answer score
         # Values over 88.72284 will overflow when passed through exponential, so logits are truncated.
-        logits[logits < -88.7] = -88.7
-        token_probabilities = 1 / (1 + np.exp(-logits)) * inputs.attention_mask
+        truncated_logits = logits.clone()
+        truncated_logits[truncated_logits < -88.7] = -88.7
+        token_probabilities = 1 / (1 + np.exp(-truncated_logits)) * inputs.attention_mask
         token_types = [
             "segment_ids",
             "column_ids",
@@ -430,10 +373,13 @@ class _TapasEncoder(_BaseTapasEncoder):
 
     def predict(self, query: str, documents: List[Document], top_k: int) -> Dict:
         answers = []
-        table_documents = self._check_documents(documents)
+        table_documents = _check_documents(documents)
         for document in table_documents:
             table: pd.DataFrame = document.content
-            model_inputs = self._preprocess(query, table, self.tokenizer, self.max_seq_len)
+            table = table.astype(str)
+            model_inputs = self.tokenizer(
+                table=table, queries=query, max_length=self.max_seq_len, return_tensors="pt", truncation=True
+            )
             model_inputs.to(self.device)
 
             current_answer = self._predict_tapas(model_inputs, document)
@@ -443,8 +389,15 @@ class _TapasEncoder(_BaseTapasEncoder):
         results = {"query": query, "answers": answers[:top_k]}
         return results
 
+    def predict_batch(self, queries: List[str], documents: List[List[Document]], top_k: int):
+        results: Dict = {"queries": queries, "answers": []}
+        for query, docs in zip(queries, documents):
+            preds = self.predict(query=query, documents=docs, top_k=top_k)
+            results["answers"].append(preds["answers"])
+        return results
 
-class _TapasScoredEncoder(_BaseTapasEncoder):
+
+class _TapasScoredEncoder:
     def __init__(
         self,
         device: torch.device,
@@ -469,10 +422,12 @@ class _TapasScoredEncoder(_BaseTapasEncoder):
         self.return_no_answer = return_no_answer
 
     def _predict_tapas_scored(self, inputs: BatchEncoding, document: Document) -> Tuple[List[Answer], float]:
-        table: pd.DataFrame = document.content
+        orig_table: pd.DataFrame = document.content
+        string_table = orig_table.astype(str)
 
         # Forward pass through model
-        with torch.no_grad():
+        self.model.eval()
+        with torch.inference_mode():
             outputs = self.model.tapas(**inputs)
             table_score = self.model.classifier(outputs.pooler_output)
 
@@ -545,8 +500,8 @@ class _TapasScoredEncoder(_BaseTapasEncoder):
         answers = []
         for answer_span_idx in top_k_answer_spans.indices:
             current_answer_span = possible_answer_spans[answer_span_idx]
-            answer_str = table.iat[current_answer_span[:2]]
-            answer_offsets = self._calculate_answer_offsets([current_answer_span[:2]], document.content)
+            answer_str = string_table.iat[current_answer_span[:2]]
+            answer_offsets = _calculate_answer_offsets([current_answer_span[:2]], string_table)
             # As the general table score is more important for the final score, it is double weighted.
             current_score = ((2 * table_relevancy_prob) + span_logits_softmax[0, answer_span_idx].item()) / 3
 
@@ -555,11 +510,11 @@ class _TapasScoredEncoder(_BaseTapasEncoder):
                     answer=answer_str,
                     type="extractive",
                     score=current_score,
-                    context=document.content,
+                    context=string_table,
                     offsets_in_document=answer_offsets,
                     offsets_in_context=answer_offsets,
-                    document_id=document.id,
-                    meta={"aggregation_operator": "NONE", "answer_cells": table.iat[current_answer_span[:2]]},
+                    document_ids=[document.id],
+                    meta={"aggregation_operator": "NONE", "answer_cells": string_table.iat[current_answer_span[:2]]},
                 )
             )
 
@@ -568,10 +523,13 @@ class _TapasScoredEncoder(_BaseTapasEncoder):
     def predict(self, query: str, documents: List[Document], top_k: int) -> Dict:
         answers = []
         no_answer_score = 1.0
-        table_documents = self._check_documents(documents)
+        table_documents = _check_documents(documents)
         for document in table_documents:
             table: pd.DataFrame = document.content
-            model_inputs = self._preprocess(query, table, self.tokenizer, self.max_seq_len)
+            table = table.astype(str)
+            model_inputs = self.tokenizer(
+                table=table, queries=query, max_length=self.max_seq_len, return_tensors="pt", truncation=True
+            )
             model_inputs.to(self.device)
 
             current_answers, current_no_answer_score = self._predict_tapas_scored(model_inputs, document)
@@ -588,7 +546,7 @@ class _TapasScoredEncoder(_BaseTapasEncoder):
                     context=None,
                     offsets_in_context=[Span(start=0, end=0)],
                     offsets_in_document=[Span(start=0, end=0)],
-                    document_id=None,
+                    document_ids=None,
                     meta=None,
                 )
             )
@@ -597,6 +555,13 @@ class _TapasScoredEncoder(_BaseTapasEncoder):
         results = {"query": query, "answers": answers[:top_k]}
         return results
 
+    def predict_batch(self, queries: List[str], documents: List[List[Document]], top_k: int):
+        results: Dict = {"queries": queries, "answers": []}
+        for query, docs in zip(queries, documents):
+            preds = self.predict(query=query, documents=docs, top_k=top_k)
+            results["answers"].append(preds["answers"])
+        return results
+
     class _TapasForScoredQA(TapasPreTrainedModel):
         def __init__(self, config):
             super().__init__(config)
@@ -682,8 +647,9 @@ class RCIReader(BaseReader):
         self.devices, _ = initialize_device_settings(use_cuda=use_gpu, multi_gpu=False)
         if len(self.devices) > 1:
             logger.warning(
-                f"Multiple devices are not supported in {self.__class__.__name__} inference, "
-                f"using the first device {self.devices[0]}."
+                "Multiple devices are not supported in %s inference, using the first device %s.",
+                self.__class__.__name__,
+                self.devices[0],
             )
 
         self.row_model = AutoModelForSequenceClassification.from_pretrained(
@@ -740,20 +706,12 @@ class RCIReader(BaseReader):
             top_k = self.top_k
 
         answers = []
-        for document in documents:
-            if document.content_type != "table":
-                logger.warning("Skipping document with id '%s' in RCIReader as it is not of type table.", document.id)
-                continue
-
-            table: pd.DataFrame = document.content
-            if table.shape[0] == 0:
-                logger.warning(
-                    "Skipping document with id '%s' in RCIReader as it does not contain any rows.", document.id
-                )
-                continue
-            table = table.astype(str)
+        table_documents = _check_documents(documents)
+        for document in table_documents:
             # Create row and column representations
-            row_reps, column_reps = self._create_row_column_representations(table)
+            table: pd.DataFrame = document.content
+            string_table = table.astype(str)
+            row_reps, column_reps = self._create_row_column_representations(string_table)
 
             # Get row logits
             row_inputs = self.row_tokenizer(
@@ -765,7 +723,10 @@ class RCIReader(BaseReader):
                 padding=True,
             )
             row_inputs.to(self.devices[0])
-            row_logits = self.row_model(**row_inputs)[0].detach().cpu().numpy()[:, 1]
+            self.row_model.eval()
+            with torch.inference_mode():
+                row_outputs = self.row_model(**row_inputs)
+            row_logits = row_outputs[0].detach().cpu().numpy()[:, 1]
 
             # Get column logits
             column_inputs = self.column_tokenizer(
@@ -777,7 +738,10 @@ class RCIReader(BaseReader):
                 padding=True,
             )
             column_inputs.to(self.devices[0])
-            column_logits = self.column_model(**column_inputs)[0].detach().cpu().numpy()[:, 1]
+            self.column_model.eval()
+            with torch.inference_mode():
+                column_outputs = self.column_model(**column_inputs)
+            column_logits = column_outputs[0].detach().cpu().numpy()[:, 1]
 
             # Calculate cell scores
             current_answers: List[Answer] = []
@@ -788,17 +752,17 @@ class RCIReader(BaseReader):
                     current_cell_score = float(row_score + col_score)
                     cell_scores_table[-1].append(current_cell_score)
 
-                    answer_str = table.iloc[row_idx, col_idx]
-                    answer_offsets = self._calculate_answer_offsets(row_idx, col_idx, table)
+                    answer_str = string_table.iloc[row_idx, col_idx]
+                    answer_offsets = self._calculate_answer_offsets(row_idx, col_idx, string_table)
                     current_answers.append(
                         Answer(
                             answer=answer_str,
                             type="extractive",
                             score=current_cell_score,
-                            context=table,
+                            context=string_table,
                             offsets_in_document=[answer_offsets],
                             offsets_in_context=[answer_offsets],
-                            document_id=document.id,
+                            document_ids=[document.id],
                         )
                     )
 
@@ -821,7 +785,7 @@ class RCIReader(BaseReader):
         column_reps = []
         columns = table.columns
 
-        for idx, row in table.iterrows():
+        for _, row in table.iterrows():
             current_row_rep = " * ".join([header + " : " + cell for header, cell in zip(columns, row)])
             row_reps.append(current_row_rep)
 
@@ -834,7 +798,7 @@ class RCIReader(BaseReader):
 
     @staticmethod
     def _calculate_answer_offsets(row_idx, column_index, table) -> Span:
-        n_rows, n_columns = table.shape
+        _, n_columns = table.shape
         answer_cell_offset = (row_idx * n_columns) + column_index
 
         return Span(start=answer_cell_offset, end=answer_cell_offset + 1)
@@ -846,41 +810,106 @@ class RCIReader(BaseReader):
         top_k: Optional[int] = None,
         batch_size: Optional[int] = None,
     ):
-        # TODO: Currently, just calls naively predict method, so there is room for improvement.
+        if top_k is None:
+            top_k = self.top_k
 
-        results: Dict = {"queries": queries, "answers": []}
+        single_doc_list = bool(len(documents) > 0 and isinstance(documents[0], Document))
 
-        single_doc_list = False
-        # Docs case 1: single list of Documents -> apply each query to all Documents
-        if len(documents) > 0 and isinstance(documents[0], Document):
-            single_doc_list = True
-            for query in queries:
-                for doc in documents:
-                    if not isinstance(doc, Document):
-                        raise HaystackError(f"doc was of type {type(doc)}, but expected a Document.")
-                    preds = self.predict(query=query, documents=[doc], top_k=top_k)
-                    results["answers"].append(preds["answers"])
+        inputs = _flatten_inputs(queries, documents)
 
-        # Docs case 2: list of lists of Documents -> apply each query to corresponding list of Documents, if queries
-        # contains only one query, apply it to each list of Documents
-        elif len(documents) > 0 and isinstance(documents[0], list):
-            if len(queries) == 1:
-                queries = queries * len(documents)
-            if len(queries) != len(documents):
-                raise HaystackError("Number of queries must be equal to number of provided Document lists.")
-            for query, cur_docs in zip(queries, documents):
-                if not isinstance(cur_docs, list):
-                    raise HaystackError(f"cur_docs was of type {type(cur_docs)}, but expected a list of Documents.")
-                preds = self.predict(query=query, documents=cur_docs, top_k=top_k)
-                results["answers"].append(preds["answers"])
+        results: Dict[str, List] = {"queries": inputs["queries"], "answers": []}
+        for query, docs in zip(inputs["queries"], inputs["docs"]):
+            preds = self.predict(query=query, documents=docs, top_k=top_k)
+            results["answers"].append(preds["answers"])
 
         # Group answers by question in case of multiple queries and single doc list
         if single_doc_list and len(queries) > 1:
-            answers_per_query = int(len(results["answers"]) / len(queries))
+            num_docs_per_query = int(len(results["answers"]) / len(queries))
             answers = []
-            for i in range(0, len(results["answers"]), answers_per_query):
-                answer_group = results["answers"][i : i + answers_per_query]
+            for i in range(0, len(results["answers"]), num_docs_per_query):
+                answer_group = results["answers"][i : i + num_docs_per_query]
                 answers.append(answer_group)
             results["answers"] = answers
 
         return results
+
+
+def _calculate_answer_offsets(answer_coordinates: List[Tuple[int, int]], table: pd.DataFrame) -> List[Span]:
+    """
+    Calculates the answer cell offsets of the linearized table based on the answer cell coordinates.
+
+    :param answer_coordinates: List of answer coordinates.
+    :param table: Table containing the answers in answer coordinates.
+    """
+    answer_offsets = []
+    _, n_columns = table.shape
+    for coord in answer_coordinates:
+        answer_cell_offset = (coord[0] * n_columns) + coord[1]
+        answer_offsets.append(Span(start=answer_cell_offset, end=answer_cell_offset + 1))
+    return answer_offsets
+
+
+def _check_documents(documents: List[Document]) -> List[Document]:
+    """
+    Check that the content type of all `documents` is of type 'table' otherwise remove that document from the list.
+
+    :param documents: List of documents to be checked.
+    """
+    table_documents = []
+    for document in documents:
+        if document.content_type != "table":
+            logger.warning("Skipping document with id '%s' in TableReader as it is not of type table.", document.id)
+            continue
+
+        table: pd.DataFrame = document.content
+        if table.shape[0] == 0:
+            logger.warning(
+                "Skipping document with id '%s' in TableReader as it does not contain any rows.", document.id
+            )
+            continue
+
+        table_documents.append(document)
+    return table_documents
+
+
+def _flatten_inputs(queries: List[str], documents: Union[List[Document], List[List[Document]]]) -> Dict[str, List]:
+    """Flatten (and copy) the queries and documents into lists of equal length.
+
+    - If you provide a list containing a single query...
+        - ... and a single list of Documents, the query will be applied to each Document individually.
+        - ... and a list of lists of Documents, the query will be applied to each list of Documents and the Answers
+          will be aggregated per Document list.
+
+    - If you provide a list of multiple queries...
+        - ... and a single list of Documents, each query will be applied to each Document individually.
+        - ... and a list of lists of Documents, each query will be applied to its corresponding list of Documents
+          and the Answers will be aggregated per query-Document pair.
+
+    :param queries: Single query string or list of queries.
+    :param documents: Single list of Documents or list of lists of Documents in which to search for the answers.
+                      Documents should be of content_type ``'table'``.
+    """
+    # Docs case 1: single list of Documents -> apply each query to all Documents
+    inputs: Dict[str, List] = {"queries": [], "docs": []}
+    if len(documents) > 0 and isinstance(documents[0], Document):
+        for query in queries:
+            for doc in documents:
+                if not isinstance(doc, Document):
+                    raise HaystackError(f"doc was of type {type(doc)}, but expected a Document.")
+                inputs["queries"].append(query)
+                inputs["docs"].append([doc])
+
+    # Docs case 2: list of lists of Documents -> apply each query to corresponding list of Documents, if queries
+    # contains only one query, apply it to each list of Documents
+    elif len(documents) > 0 and isinstance(documents[0], list):
+        total_queries = queries.copy()
+        if len(total_queries) == 1:
+            total_queries = queries * len(documents)
+        if len(total_queries) != len(documents):
+            raise HaystackError("Number of queries must be equal to number of provided Document lists.")
+        for query, cur_docs in zip(total_queries, documents):
+            if not isinstance(cur_docs, list):
+                raise HaystackError(f"cur_docs was of type {type(cur_docs)}, but expected a list of Documents.")
+            inputs["queries"].append(query)
+            inputs["docs"].append(cur_docs)
+    return inputs
