@@ -31,8 +31,11 @@ user_id: Optional[str] = None
 logger = logging.getLogger(__name__)
 
 # disable posthog logging
-logging.getLogger("posthog").setLevel(CRITICAL)
-logging.getLogger("backoff").setLevel(CRITICAL)
+for module_name in ["posthog", "backoff"]:
+    logging.getLogger(module_name).setLevel(CRITICAL)
+    # Prevent module from sending errors to stderr when an exception is encountered during an emit() call
+    logging.getLogger(module_name).addHandler(logging.NullHandler())
+    logging.getLogger(module_name).propagate = False
 
 
 class TelemetryFileType(Enum):
@@ -130,7 +133,7 @@ def send_event(func):
     return wrapper
 
 
-def send_custom_event(event: str = "", payload: Dict[str, Any] = {}):
+def send_custom_event(event: str = "", payload: Optional[Dict[str, Any]] = None):
     """
     This method can be called directly from anywhere in Haystack to send an event.
     Enriches the given event with metadata and sends it to the posthog server if telemetry is enabled.
@@ -140,6 +143,8 @@ def send_custom_event(event: str = "", payload: Dict[str, Any] = {}):
     :param payload: A dictionary containing event meta data, e.g., parameter settings
     """
     global user_id  # pylint: disable=global-statement
+    if payload is None:
+        payload = {}
     try:
 
         def send_request(payload: Dict[str, Any]):
@@ -180,7 +185,6 @@ def send_custom_event(event: str = "", payload: Dict[str, Any] = {}):
 def send_tutorial_event(url: str):
     """
     Can be called when a tutorial dataset is downloaded so that the dataset URL is used to identify the tutorial and send an event.
-
     :param url: URL of the dataset that is loaded in the tutorial.
     """
     dataset_url_to_tutorial = {
@@ -208,6 +212,14 @@ def send_tutorial_event(url: str):
     send_custom_event(event=f"tutorial {dataset_url_to_tutorial.get(url, '?')} executed")
 
 
+def tutorial_running(tutorial_id: int):
+    """
+    Can be called when a tutorial is executed so that the tutorial_id is used to identify the tutorial and send an event.
+    :param tutorial_id: ID number of the tutorial
+    """
+    send_custom_event(event=f"tutorial {tutorial_id} executed")
+
+
 def _get_or_create_user_id() -> Optional[str]:
     """
     Randomly generates a user id or loads the id defined in the config file and returns it.
@@ -250,7 +262,11 @@ def _write_telemetry_config():
         # show a log message if telemetry config is written for the first time
         if not CONFIG_PATH.is_file():
             logger.info(
-                f"Haystack sends anonymous usage data to understand the actual usage and steer dev efforts towards features that are most meaningful to users. You can opt-out at anytime by calling disable_telemetry() or by manually setting the environment variable HAYSTACK_TELEMETRY_ENABLED as described for different operating systems on the documentation page. More information at https://docs.haystack.deepset.ai/docs/telemetry"
+                "Haystack sends anonymous usage data to understand the actual usage and steer dev efforts "
+                "towards features that are most meaningful to users. You can opt-out at anytime by calling "
+                "disable_telemetry() or by manually setting the environment variable  "
+                "HAYSTACK_TELEMETRY_ENABLED as described for different operating systems on the documentation "
+                "page. More information at https://docs.haystack.deepset.ai/docs/telemetry"
             )
             CONFIG_PATH.parents[0].mkdir(parents=True, exist_ok=True)
         user_id = _get_or_create_user_id()
